/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

/*
 * $Revision: 1.24 $
 */

/*
 * This file contains DMI definetions common to SysInfo.
 *
 * References:
 * 	http://www.dmtf.org/standards/standard_dmi.php
 *	System Management BIOS Reference Specification - Version 2.3.1
 */

#ifndef __mydmi_h__
#define __mydmi_h__ 

#include "dmideclare.h"

/*
 * Pathnames
 */
#ifndef _PATH_DEV_MEM
#define _PATH_DEV_MEM		"/dev/mem"
#endif

/*
 * Range in memory where SMBIOS data is stored.
 */
#define SMBIOS_ADDR_START	0x00E0000L	/* Start of range */
#define SMBIOS_ADDR_END		0x000FFFFF	/* End of range */
#define SMBIOS_BLOCK_SIZE	16		/* Size of each block */

/*
 * This macro is needed for all Word_t or larger variables to insure
 * the compiler does not increase allocated variable space for alignment
 * reasons and thus invalidate the variable location.
 */
#if	defined(HAVE_ATTRIBUTE_PACKED)
# define _PACK_			__attribute__ ((packed))
#else
# define _PACK_
#endif

/*
 * Basic types
 */
typedef unsigned char		Byte_t;		/* 1 byte/8 bit  */
typedef unsigned short		Word_t;		/* 2 byte/16 bit */
typedef unsigned int		Dword_t;	/* 4 byte/32 bit */

/*
 * DMI SMBIOS Entry Point Structure (EPS)
 */
typedef union {
    int			Version;		/* Which EPS version to use */
    /*
     * DMI v1 Entry Point Structure (EPS)
     */
    struct {
	/* Offset 00h */
	Byte_t		Anchor[5];		/* The string "_DMI_" */
	/* Offset 01h */
	Byte_t		Checksum;		/* Checksum of EPS */
	/* Offset 02h */
	Word_t		TableLen _PACK_;	/* Table length of TableAddr */
	/* Offset 04h */
	Dword_t		TableAddr _PACK_;	/* Address where table starts*/
	/* Offset 08h */
	Word_t		NumStructs _PACK_;	/* # of structures */
	/* Offset 10h */
#if 	defined(_BIT_FIELDS_LTOH)
	Byte_t		MinVer : 4;		/* SMBIOS minor version */
	Byte_t		MajVer : 4;		/* SMBIOS major version */
#else
	Byte_t		MajVer : 4;		/* SMBIOS major version */
	Byte_t		MinVer : 4;		/* SMBIOS minor version */
#endif
    } v1;
    /*
     * DMI v2 Entry Point Structure (EPS)
     */
    struct {
	/* Offset 00h */
	Byte_t		Anchor[4];		/* The string "_SM_" */
	/* Offset 04h */
	Byte_t		Checksum;		/* Checksum of EPS */
	/* Offset 05h */
	Byte_t		Length;			/* Length of EPS */
	/* Offset 06h */
	Byte_t		MajVer;			/* SMBIOS major version */
	/* Offset 07h */
	Byte_t		MinVer;			/* SMBIOS minor version */
	/* Offset 08h */
	Word_t		MaxSize _PACK_;		/* Max size largest struct */
	/* Offset 0Ah */
	Byte_t		EPrev;			/* Entry Point Revision */
	/* Offset 0B-0Fh */
	Byte_t		Data[5];		/* EP specific data */
	/* Offset 10h */
	Byte_t		InterAnchor[5];		/* The string "_DMI_" */
	/* Offset 15h */
	Byte_t		InterChecksum;		/* Intermediate checksum */
	/* Offset 16h */
	Word_t		TableLen _PACK_;	/* Table length of TableAddr */
	/* Offset 18h */
	Dword_t		TableAddr _PACK_;	/* Address where table starts*/
	/* Offset 1Ch */
	Word_t		NumStructs _PACK_;	/* # of structures */
	/* Offset 1Eh */
	Byte_t		BCDrev;			/* BCD revision */
    } v2;
} DmiEps_t;

/*
 * Given an EPS (e), return member (m) depending on the EPS version
 */
#define EPS(e,m)	( (e->Version == 1) ? e->v1.m : e->v2.m )

/*
 * DMI Header for each structure
 */
typedef struct {
    /* Offset 00h */
    Byte_t		Type;			/* Type of structure */
    /* Offset 01h */
    Byte_t		Length;			/* Length of structure */
    /* Offset 02h */
    Word_t		Handle _PACK_;		/* Unique 16-bit handle */
} DmiHeader_t;

/*
 * DMI Structure types.
 * Values present in DmiHeader_t.Type
 */
#define DMI_STRUCT_BIOS		0
#define DMI_STRUCT_SYSINFO	1
#define DMI_STRUCT_BOARD	2
#define DMI_STRUCT_ENCLOSURE	3
#define DMI_STRUCT_PROC		4
#define DMI_STRUCT_MEMCTL	5		/* Obsoleted by Type 16 */
#define DMI_STRUCT_MEMMOD	6		/* Obsoleted by Type 17 */
#define DMI_STRUCT_CACHE	7
#define DMI_STRUCT_PORT		8
#define DMI_STRUCT_SLOT		9
#define DMI_STRUCT_ONBOARD	10
#define DMI_STRUCT_OEM		11
#define DMI_STRUCT_SYSCONFIG	12
#define DMI_STRUCT_BIOSLANG	13
#define DMI_STRUCT_GROUP	14
#define DMI_STRUCT_EVLOG	15
#define DMI_STRUCT_MEMARRAY	16
#define DMI_STRUCT_MEMDEV	17
#define DMI_STRUCT_MEMERR32	18
#define DMI_STRUCT_MEMARRMAP	19
#define DMI_STRUCT_MEMDEVMAP	20
#define DMI_STRUCT_POINTER	21
#define DMI_STRUCT_BATTERY	22
#define DMI_STRUCT_SYSRESET	23
#define DMI_STRUCT_HWSECURITY	24
#define DMI_STRUCT_SYSPWRCTL	25
#define DMI_STRUCT_VOLTPROBE	26
#define DMI_STRUCT_COOLDEV	27
#define DMI_STRUCT_TEMPPROBE	28
#define DMI_STRUCT_ELECTPROBE	29
#define DMI_STRUCT_OOBACCESS	30
#define DMI_STRUCT_BIS		31
#define DMI_STRUCT_SYSBOOT	32
#define DMI_STRUCT_MEMERR64	33
#define DMI_STRUCT_MGTDEV	34
#define DMI_STRUCT_MGTDEVCOM	35
#define DMI_STRUCT_MGTDEVTHR	36
#define DMI_STRUCT_MEMCHAN	37
#define DMI_STRUCT_IPMI		38
#define DMI_STRUCT_SYSPWRSUP	39
#define DMI_STRUCT_INACTIVE	126
#define DMI_STRUCT_ENDTABLE	127

/*
 * BIOS Information structure (Type 0) - DMI_STRUCT_BIOS
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Vendor;			/* (String) Vendor's Name */
    /* Offset 05h */
    Byte_t		Version;		/* (String) Version */
    /* Offset 06h */
    Word_t		StartAddr _PACK_;	/* BIOS starting address */
    /* Offset 08h */
    Byte_t		ReleaseDate;		/* (String) Release date */
    /* Offset 09h */
    Byte_t		RomSize;		/* BIOS ROM size 64K blocks */
#if 	defined(_BIT_FIELDS_LTOH)
    /* Offset 0Ah */
    Dword_t		_reserved0	: 1,
    			_reserved1	: 1,
    			_reserved2	: 1,
    			NoChars		: 1,	/* Characteristics unsupp. */
    			ISA		: 1,	/* supported? */
    			MCA		: 1,	/* supported? */
    			EISA		: 1,	/* supported? */
    			PCI		: 1,	/* supported? */
    			PCCard		: 1,	/* supported? */
    			PnP		: 1,	/* Plug and Play */
    			APM		: 1,	/* supported? */
    			FlashUpgrade	: 1,	/* BIOS is flash upgradable */
    			Shadow		: 1,	/* BIOS shadowing */
    			VLVESA		: 1,	/* supported? */
    			ESCD		: 1,	/* supported? */
    			CDboot		: 1,	/* Boot from CD */
    			SelectBoot	: 1,	/* Selectable boot */
    			ROMsocketed	: 1,	/* ROM is socketed */
    			PCCardBoot	: 1,	/* Boot from PC Card */
    			EDD		: 1,	/* supported? */
    			JpnNEC9800	: 1,	/* Japanese NEC 9800 */
    			JpnToshiba	: 1,	/* Japanese Toshiba */
    			Flp5in360k	: 1,	/* 5.25in 360k floppy */
    			Flp5in12m	: 1,	/* 5.25in 1.2m floppy */
    			Flp3in720k	: 1,	/* 3.5in 720k floppy */
    			Flp3in288m	: 1,	/* 3.5in 2.88m floppy */
    			PrintScreen	: 1,	/* supported? */
    			Kbd8042		: 1,	/* 8042 keyboard */
    			Serial		: 1,	/* supported? */
    			Printer		: 1,	/* supported? */
    			CGA		: 1,	/* supported? */
    			NecPC98		: 1	/* NEC PC98 */
	_PACK_;
    /* Offset 10h */
    Word_t		BiosVendor _PACK_;	/* Reserved for BIOS Vendor */
    /* Offset 11h */
    Word_t		SysVendor _PACK_;	/* Reserved for Sys Vendor */
    /* Offset 12h */
    Byte_t		ACPI		: 1,	/* supported */
    			USBlegacy	: 1,	/* supported */
    			AGP		: 1,	/* supported */
    			I2Oboot		: 1,	/* supported */
    			LS120boot	: 1,	/* supported */
    			AtapiZipBoot	: 1,	/* supported */
    			Boot1394	: 1,	/* supported */
    			SmartBattery	: 1;	/* supported */
    /* Offset 13h */
    Byte_t		BootSpec	: 1,	/* BIOS Boot Specification */
    			FuncKeyNetBoot	: 1,	/* Function Key Net Boot */
    			_reserved13	: 6;
#elif	defined(_BIT_FIELDS_HTOL)
    /* 
     * We shouldn't need to define HTOL values since DMI is only available
     * on little indian (x86) currently.
     */
#error "No values are defined for _BIT_FIELDS_HTOL systems"
#endif	/* _BIT_FIELDS_ */
} DmiStructBios_t;

/*
 * System Information (Type 1) - DMI_STRUCT_SYSINFO
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Mfg;			/* (String) Manufacturer     */
    /* Offset 05h */
    Byte_t		Product;		/* (String) Product Name     */
    /* Offset 06h */
    Byte_t		Version;		/* (String) Version          */
    /* Offset 07h */
    Byte_t		Serial;			/* (String) Serial #         */
    /* Offset 08h */
    Byte_t		UUID[16];		/* Universal Unique ID       */
    						/* FFh=not present, settable */
    						/* 00h=not present           */
    /* Offset 18h */
    Byte_t		WakeUp;			/* Wake Up type              */
} DmiStructSysInfo_t;

/*
 * Values for DmiStructSysInfo_t.WakeUp
 */
#define DMI_WAKEUP_APM		0x03		/* APM Timer                 */
#define DMI_WAKEUP_MODEM	0x04		/* Modem Ring                */
#define DMI_WAKEUP_LAN		0x05		/* LAN Remote                */
#define DMI_WAKEUP_PWRSW	0x06		/* Power Switch              */
#define DMI_WAKEUP_PCIPME	0x07		/* PCI PME#                  */
#define DMI_WAKEUP_ACPWR	0x08		/* AC Power Restored         */

/*
 * Board Information (Type 2) - DMI_STRUCT_BOARD
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Mfg;			/* (String) Manufacturer     */
    /* Offset 05h */
    Byte_t		Product;		/* (String) Product Name     */
    /* Offset 06h */
    Byte_t		Version;		/* (String) Version          */
    /* Offset 07h */
    Byte_t		Serial;			/* (String) Serial #         */
} DmiStructBoard_t;

/*
 * System Enclosure (Chassis) Information (Type 3) - DMI_STRUCT_ENCLOSURE
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Mfg;			/* (String) Manufacturer     */
    /* Offset 05h */
#if 	defined(_BIT_FIELDS_LTOH)
    Byte_t		Type		: 7;
    Byte_t		Lockable 	: 1;	/* Chassis lock present      */
#else	/* _BIT_FIELDS_HTOL */
#error "No values set for _BIT_FIELDS_HTOL"
#endif	/* _BIT_FIELDS_LTOH */
    /* Offset 06h */
    Byte_t		Version;		/* (String) Version          */
    /* Offset 07h */
    Byte_t		Serial;			/* (String) Serial #         */
    /* Offset 08h */
    Byte_t		AssetTag;		/* (String) Asset Tag #      */
    /* Offset 09h */
    Byte_t		BootUpState;		/* Boot up state             */
    /* Offset 0Ah */
    Byte_t		PwrSupplyState;		/* Power Supply State        */
    /* Offset 0Bh */
    Byte_t		ThermalState;		/* Thermal state at boot     */
    /* Offset 0Ch */
    Byte_t		SecurityStatus;		/* Physical Security Status  */
    /* Offset 0Dh */
    Dword_t		Oem _PACK_;		/* OEM specific              */
} DmiStructEnclosure_t;

/*
 * Processor Information (Type 4) - DMI_STRUCT_PROC
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Socket;			/* (String) Socket location  */
    /* Offset 05h */
    Byte_t		Type;			/* Processor Type            */
    /* Offset 06h */
    Byte_t		Family;			/* Processor Family          */
    /* Offset 07h */
    Byte_t		Mfg;			/* (String) Manufacturer     */
    /* Offset 08h */
    Byte_t		CpuId[8];		/* Processor ID              */
    /* Offset 10h */
    Byte_t		Version;		/* (String) Version          */
    /* Offset 11h */
    Byte_t		Voltage;		/* Voltage	             */
    /* Offset 12h */
    Word_t		ExtClockFreq _PACK_;	/* External Clock Freq (MHz) */
    /* Offset 14h */
    Word_t		MaxSpeed _PACK_;	/* Max int CPU speed (Mhz)   */
    /* Offset 16h */
    Word_t		CurSpeed _PACK_;	/* Current CPU speed (Mhz)   */
    /* Offset 18h */
#if 	defined(_BIT_FIELDS_LTOH)
    Byte_t		Status		: 3,	/* CPU Status                */
    			_reserved18_1	: 3,	/* Reserved                  */
    			Present		: 1,	/* CPU Socket occupied       */
    			_reserved18_2	: 1;	/* Reserved                  */
#else	/* _BIT_FIELDS_HTOL */
#error "No values set for _BIT_FIELDS_HTOL"
#endif	/* _BIT_FIELDS_LTOH */
    /* Offset 19h */
    Byte_t		Upgrade;		/* Upgrade                   */
    /* Offset 1Ah */
    Word_t		L1handle _PACK_;	/* L1 cache DMI handle       */
    /* Offset 1Ch */
    Word_t		L2handle _PACK_;	/* L2 cache DMI handle       */
    /* Offset 1Eh */
    Word_t		L3handle _PACK_;	/* L3 cache DMI handle       */
    /* Offset 20h */
    Byte_t		Serial;			/* (String) Serial #         */
    /* Offset 21h */
    Byte_t		AssetTag;		/* (String) Asset Tag        */
    /* Offset 22h */
    Byte_t		Part;			/* (String) Part #           */
} DmiStructProc_t;

/*
 * Values for DmiStructProc_t.Type
 * See section 3.3.5.1 of SMBIOS Ref Spec.
 */
#define DMI_PROC_TYPE_CPU	0x03
#define DMI_PROC_TYPE_MATH	0x04
#define DMI_PROC_TYPE_DSP	0x05
#define DMI_PROC_TYPE_VIDEO	0x06

/*
 * Memory Controller (Type 5) - DMI_STRUCT_MEMCTL
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		ErrDetect;		/* Error Detection Method    */
    /* Offset 05h */
#if 	defined(_BIT_FIELDS_LTOH)
    Byte_t		EccOther	: 1,	/* Other                     */
    			EccUnknown	: 1,	/* Unknown                   */
    			EccNone		: 1,	/* None                      */
    			EccSingle	: 1,	/* Single Bit                */
    			EccDouble	: 1,	/* Double Bit                */
    			EccScrub	: 1,	/* Error Scrubbing           */
    			_EccReserved	: 2;
#elif	defined(_BIT_FIELDS_HTOL)
#error "No values are defined for _BIT_FIELDS_HTOL systems"
#endif	/* _BIT_FIELDS_ */
    /* Offset 06h */
    Byte_t		SupInter;		/* Supported Interleave      */
    /* Offset 07h */
    Byte_t		CurInter;		/* Current Interleave        */
    /* Offset 08h */
    Byte_t		MaxModSize;		/* Max supported size (MB)   */
#if 	defined(_BIT_FIELDS_LTOH)
    /* Offset 09h */
    Word_t		SupSpeedOther	: 1,	/* Supported Speed           */
    			SupSpeedUnknown	: 1,	/* Supported Speed           */
    			SupSpeed70	: 1,	/* 70ns                      */
    			SupSpeed60	: 1,	/* 60ns                      */
    			SupSpeed50	: 1,	/* 50ns                      */
    			_SupSpeedReserved:11
	_PACK_;
    /* Offset 0Bh */
    Word_t		TypeOther	: 1,	/* Supported Memory Types    */
    			TypeUnknown	: 1,	/*                           */
    			TypeStandard	: 1,	/* Standard                  */
    			TypeFPM		: 1,	/* Fast Page Mode            */
    			TypeEDO		: 1,	/* EDO                       */
    			TypeParity	: 1,	/* Parity                    */
    			TypeECC		: 1,	/* ECC                       */
    			TypeSIMM	: 1,	/* SIMM                      */
    			TypeDIMM	: 1,	/* DIMM                      */
    			TypeBurstEDO	: 1,	/* Burst EDO                 */
    			TypeSDRAM	: 1,	/* SDRAM                     */
    			_TypeReserved	: 5
	_PACK_;
    /* Offset 0Dh */
    Byte_t		Volt5		: 1,	/* 5V                        */
    			Volt33		: 1,	/* 3.3V                      */
    			Volt29		: 1,	/* 2.9V                      */
    			_VoltReserved	: 5;	/*                           */
#else	/* _BIT_FIELDS_HTOL */
#error "No values set for _BIT_FIELDS_HTOL"
#endif	/* _BIT_FIELDS_LTOH */
    /* Offset 0Eh */
    Byte_t		Slots;			/* # of Associated modules   */
    /* Offset 0Fh */
    Byte_t		Handles;		/* Start of associated mods  */
} DmiStructMemctl_t;

/*
 * Memory Module (Type 6) - DMI_STRUCT_MEMMOD
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Location;		/* (String) Socket Location  */
    /* Offset 05h */
    Byte_t		BankCon;		/* Bank connections          */
    /* Offset 06h */
    Byte_t		CurSpeed;		/* Current Speed (ns)        */
#if 	defined(_BIT_FIELDS_LTOH)
    /* Offset 07h */
    Word_t		TypeOther	: 1,	/* Supported Memory Types    */
    			TypeUnknown	: 1,	/*                           */
    			TypeStandard	: 1,	/* Standard                  */
    			TypeFPM		: 1,	/* Fast Page Mode            */
    			TypeEDO		: 1,	/* EDO                       */
    			TypeParity	: 1,	/* Parity                    */
    			TypeECC		: 1,	/* ECC                       */
    			TypeSIMM	: 1,	/* SIMM                      */
    			TypeDIMM	: 1,	/* DIMM                      */
    			TypeBurstEDO	: 1,	/* Burst EDO                 */
    			TypeSDRAM	: 1,	/* SDRAM                     */
    			_TypeReserved	: 5
	_PACK_;
    /* Offset 09h */
    Byte_t		InstSize	: 7,	/* Installed Size            */
    			InstSizeCon	: 1;	/* 0=Single 1=Double         */
    /* Offset 0Ah */
    Byte_t		EnabledSize	: 7,	/* Enabled Size              */
   			EnabledSizeCon	: 1;	/* 0=Single 1=Double         */
#else	/* _BIT_FIELDS_HTOL */
#error "No values set for _BIT_FIELDS_HTOL"
#endif	/* _BIT_FIELDS_LTOH */
    /* Offset 0Bh */
    Byte_t		ErrStatus;		/* Error Status		     */
} DmiStructMemmod_t;

/*
 * Cache Information (Type 7) - DMI_STRUCT_CACHE
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Socket;			/* (String) Socket ref       */
#if 	defined(_BIT_FIELDS_LTOH)
    /* Offset 05h */
    Word_t		Level		: 3,	/* Cache Level		     */
    			Socketed	: 1,	/* Is Socketed?		     */
    			_reserved1	: 1,
    			Location	: 2,	/* Location relative to CPU  */
    			Enabled		: 1,	/* Enabled/disabled?	     */
    			OpMode		: 2,	/* Operational mode	     */
    			_reserved2	: 6
	_PACK_;
    /* Offset 07h */
    Word_t		MaxSize		: 15,	/* Max size (in Step sizes)  */
    			MaxSizeStep	: 1	/* 0=1K 1=64K                */
	_PACK_;
    /* Offset 09h */
    Word_t		InstSize	: 15,	/* Max size (in Step sizes)  */
    			InstSizeStep	: 1	/* 0=1K 1=64K                */
	_PACK_;
    /* Offset 0Bh */				/* Supported SRAM Type       */
    Word_t		SupSramOther	: 1,	/* Other                     */
    			SupSramUnknown	: 1,	/* Unknown                   */
    			SupSramNonBurst	: 1,	/* Non-Burst                 */
    			SupSramPipeBurst: 1,	/* Pipeline Burst            */
    			SupSramSync	: 1,	/* Syncronous                */
    			SupSramAsync	: 1,	/* Asyncronous               */
    			_reserved0B	: 10
	_PACK_;
    /* Offset 0Dh */				/* Current SRAM Type         */
    Word_t		CurSramOther	: 1,	/* Other                     */
    			CurSramUnknown	: 1,	/* Unknown                   */
    			CurSramNonBurst	: 1,	/* Non-Burst                 */
    			CurSramPipeBurst: 1,	/* Pipeline Burst            */
    			CurSramSync	: 1,	/* Syncronous                */
    			CurSramAsync	: 1,	/* Asyncronous               */
    			_reserved0D	: 10
	_PACK_;
#else	/* _BIT_FIELDS_HTOL */
#error "No values set for _BIT_FIELDS_HTOL"
#endif	/* _BIT_FIELDS_LTOH */
    /* Offset 0Fh */
    Byte_t		Speed;			/* Speed (ns)                */
    /* Offset 10h */
    Byte_t		EccType;		/* ECC Type		     */
    /* Offset 11h */
    Byte_t		SysType;		/* System Cache Type         */
    /* Offset 12h */
    Byte_t		Associativity;		/* Associativity             */
} DmiStructCache_t;

/*
 * Port Information (Type 8) - DMI_STRUCT_PORT
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		IntLocation;		/* (String) Intern Location  */
    /* Offset 05h */
    Byte_t		IntConn;		/* Internal Connector Type   */
    /* Offset 06h */
    Byte_t		ExtLocation;		/* (String) Extern Location  */
    /* Offset 07h */
    Byte_t		ExtConn;		/* Internal Connector Type   */
    /* Offset 08h */
    Byte_t		Type;			/* Type of Port              */
} DmiStructPort_t;

/*
 * System Slots (Type 9) - DMI_STRUCT_SLOT
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Designation;		/* (String) Designation      */
    /* Offset 05h */
    Byte_t		Type;			/* Slot Type                 */
    /* Offset 06h */
    Byte_t		BusWidth;		/* Data Bus Width            */
    /* Offset 07h */
    Byte_t		Usage;			/* Current Usage             */
    /* Offset 08h */
    Byte_t		Length;			/* Slot length               */
    /* Offset 09h */
    Word_t		ID _PACK_;		/* ID                        */
#if 	defined(_BIT_FIELDS_LTOH)
    /* Offset 0Bh */
    Byte_t		CharUnknown	: 1,	/* Characteristics unknown   */
			v50		: 1,	/* 5.0 volts                 */
			v33		: 1,	/* 3.3 volts                 */
			SlotShared	: 1,	/* Slot open shared          */
			SupPCCard16	: 1,	/* Supports PC Card-16       */
			SupCardBus	: 1,	/* Supports CardBus          */
			SupZoomVid	: 1,	/* Supports Zoom Video       */
			ModemRingRes	: 1;	/* Supports modem ring resume*/
    /* Offset 0Ch */
    Byte_t		SupPME		: 1,	/* Supports Pwr Mgt Enable   */
			SupHotPlug	: 1,	/* Supports Hot-Plug Devs    */
			SupSMBus	: 1,	/* Supports SMBus            */
			_reserved0C	: 5;
#else	/* _BIT_FIELDS_HTOL */
#error "No values set for _BIT_FIELDS_HTOL"
#endif	/* _BIT_FIELDS_LTOH */

} DmiStructSlot_t;

/*
 * OEM Info (Type 11) - DMI_STRUCT_OEM
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Count;			/* # of OEM Strings          */
} DmiStructOem_t;

/*
 * System Config Info (Type 12) - DMI_STRUCT_SYSCONFIG
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Count;			/* # of Strings              */
} DmiStructSysconfig_t;

/*
 * Group Associations (Type 14) - DMI_STRUCT_GROUP
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Name;			/* (String) Name of group    */
    /* Offset 05h */
    Byte_t		ItemType;		/* Item Type of this member  */
    /* Offset 06h */
    Word_t		ItemHandle _PACK_;	/* Item Handle               */
} DmiStructGroup_t;

/*
 * Physical Memory Array (Type 16) - DMI_STRUCT_MEMARRAY
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Location;		/* Physical location         */
    /* Offset 05h */
    Byte_t		Use;			/* Functional use            */
    /* Offset 06h */
    Byte_t		ErrCorrect;		/* Primary error correction  */
    /* Offset 07h */
    Dword_t		Capacity _PACK_;	/* Max capacity (KB)         */
    /* Offset 0Bh */
    Word_t		ErrHandle _PACK_;	/* Error Info Handle         */
    /* Offset 0Dh */
    Word_t		Count _PACK_;		/* # of memory devices       */
} DmiStructMemarray_t;

/*
 * Physical Memory Device (Type 17) - DMI_STRUCT_MEMDEV
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Word_t		ArrayHandle _PACK_;	/* Associated Array Handle   */
    /* Offset 06h */
    Word_t		ErrHandle _PACK_;	/* Memory Error Info Handle  */
    /* Offset 08h */
    Word_t		TotalWidth _PACK_;	/* Total width (bits)        */
    /* Offset 0Ah */
    Word_t		DataWidth _PACK_;	/* Data width (bits)         */
#if 	defined(_BIT_FIELDS_LTOH)
    /* Offset 0Ch */
    Word_t		Size		: 15,	/* Total width (bits)        */
			SizeGran	: 1	/* 0=MB 1=KB		     */
	_PACK_;
#else	/* _BIT_FIELDS_HTOL */
#error "No values set for _BIT_FIELDS_HTOL"
#endif	/* _BIT_FIELDS_LTOH */
    /* Offset 0Eh */
    Byte_t		FormFactor;		/* Form Factor   	     */
    /* Offset 0Fh */
    Byte_t		DevSet;			/* Device Set		     */
    /* Offset 10h */
    Byte_t		DevLocator;		/* (String) Device Locator   */
    /* Offset 11h */
    Byte_t		BankLocator;		/* (String) Bank Locator     */
    /* Offset 12h */
    Byte_t		Type;			/* Type of memory	     */
#if 	defined(_BIT_FIELDS_LTOH)
    /* Offset 13h */
    Word_t		_reserved13a	: 1,	/* Type detail		     */
			TypeOther	: 1,
			TypeUnknown	: 1,
			TypeFP		: 1,
			TypeStatic	: 1,
			TypePseudo	: 1,
			TypeRAMBUS	: 1,
			TypeSync	: 1,
			TypeCMOS	: 1,
			TypeEDO		: 1,
			TypeWinDRAM	: 1,
			TypeCacheDRAM	: 1,
			TypeNV		: 1,
			_reserved13b	: 3
	_PACK_;
#else	/* _BIT_FIELDS_HTOL */
#error "No values set for _BIT_FIELDS_HTOL"
#endif	/* _BIT_FIELDS_LTOH */
    /* Offset 15h */
    Word_t		Speed _PACK_;		/* Speed (MHz)		     */
    /* Offset 17h */
    Byte_t		Mfg;			/* (String) Manufacturer     */
    /* Offset 18h */
    Byte_t		Serial;			/* (String) Serial #         */
    /* Offset 19h */
    Byte_t		Asset;			/* (String) Asset Tag        */
    /* Offset 1Ah */
    Byte_t		Part;			/* (String) Part #           */
} DmiStructMemdev_t;

/*
 * Built-in Pointing Device (Type 21) - DMI_STRUCT_POINTER
 */
typedef struct {
    /* Offset 00-03h */
    DmiHeader_t		Header;
    /* Offset 04h */
    Byte_t		Type;			/* Type of device            */
    /* Offset 05h */
    Byte_t		Interface;		/* Interface                 */
    /* Offset 06h */
    Byte_t		Buttons;		/* # of buttons              */
} DmiStructPointer_t;

#ifdef notdef	/* We don't actually need these right now */
/*
 * SMBIOS commands
 */
#define SMB_GET_DMI_INFORMATION	0x50	/* Return # of structs, etc 	     */
#define SMB_GET_DMI_STRUCTURE	0x51	/* Return specified struct 	     */
#define SMB_SET_DMI_STRUCTURE	0x52

/*
 * DMI Error Return Codes
 */
#define DMI_SUCCESS		0x00	/* Successful */
#define DMI_UNKNOWN_FUNCTION	0x81	/* Unknown/invalid function # passed */
#define DMI_FUNC_NOT_SUPPORT	0x82	/* Function not supported */
#define DMI_INVALID_HANDLE	0x83	/* SMBIOS struct #/handle invalid */
#define DMI_BAD_PARAM		0x84	/* Invalid parameter */
#define DMI_INVALID_SUBFUNC	0x85	/* Invalid SubFunction param */
#define DMI_NO_CHANGE		0x86	/* No changed structs */
#define DMI_ADD_STRUCT_FAIL	0x87	/* Insufficient storage space to add */
#define DMI_READ_ONLY		0x80	/* Something is read-only */
#define DMI_LOCK_NOT_SUP	0x90	/* GPNV func don't support locking */
#define DMI_LOCKED		0x91	/* GPNV is already locked */
#define DMI_INVALID_LOCK	0x92	/* GPNV lock invalid */

#endif	/* notdef */

/*
 * DMI general info
 *
 * Used internal to Dmi*() functions
 */
typedef struct {
    int				SmbiosVersion;		/* SMBIOS version */
    int				SmbiosMajVer;		/* SMBIOS major ver */
    int				SmbiosMinVer;		/* SMBIOS minor ver */
    int				SmbiosEpsRev;		/* SMBIOS revision */
    int				PnpBios;		/* PnP BIOS present */
    int				SysId;			/* SYSID present */
    DmiEps_t		       *Eps;			/* EPS data */
    void		       *Table;			/* DMI table */
    /* Internal use only */
    int			        _FileDesc;		/* File descriptor */
    DevInfo_t		      **_DevInfoTreePtr;	/* DevInfo Tree */
} Dmi_t;

/*
 * DMI Query
 *
 * Used internal to Dmi*() functions
 */
typedef struct {
    /* Input */
    int				Type;			/* DMI Type          */
    /* Results */
    char		       *ValStr1;		/* Value String 1    */
    char		       *ValStr2;		/* Value String 2    */
    char		       *ValStr3;		/* Value String 3    */
    char		       *ValStr4;		/* Value String 4    */
    char		       *ValStr5;		/* Value String 5    */
    int				ValInt1;		/* Value integer 1   */
    int				ValInt2;		/* Value integer 2   */
    int				ValInt3;		/* Value integer 3   */
} DmiQuery_t;

#endif	/* __mydmi_h__ */
