/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.10 $";
#endif

/*
 * Printer LPR functions.
 *
 * This works on most LPR/printcap based systems as well as Solaris 2.x
 */

#include "defs.h"
#if HAVE_SETJMP_H
# include <setjmp.h>
static jmp_buf			Env;
#endif

/*
 * List of printcap files to parse
 */
static char *PrintCapFiles[] = {
    "/etc/printcap", "/etc/printers.conf", NULL
};

/*
 * List of ypcat path locations
 */
static char *YpcatPath[] = {
    "/bin/ypcat", "/usr/bin/ypcat", NULL
};

/*
 * Called when the alarm goes off
 */
static void TimeOutHandler(SigNum)
     int			SigNum;
{
    if (SigNum == SIGALRM)
	SImsg(SIM_DBG,
	      "TimeOutHandler: A time-out has occured executing a comamnd.");
    else
	SImsg(SIM_DBG,
	      "TimeOutHandler: Cmd Received signal %s",
	      SignalName(SigNum));

#if	defined(HAVE_LONGJMP)
    longjmp(Env, SigNum);
#endif	/* HAVE_LONGJMP */
}

/*
 * Read the next cap entry into Buff from file pointer Fp.
 *
 * Return 1 on success.
 * Return 0 on EOF.
 * Return -1 on error.
 */
static int CapGetEnt(Buff, BuffSize, Fp)
     char		       *Buff;
     size_t			BuffSize;
     FILE		       *Fp;
{
    char		       *bp = Buff;
    size_t			bs = BuffSize;
    int				Len;

    while (fgets(bp, bs, Fp)) {
	if (bp[0] == '#' || bp[0] == '\n')
	    continue;
	Len = strlen(bp);
	if (bp[Len-2] != '\\')
	    /* No continue char at end of line so we're done */
	    return 1;
	/* Update our pointer/size */
	bp += Len;
	bs -= Len;
    }

    return 0;
}

/*
 * Given a printcap Entry, set the Queue name, aliases, and Desc
 */
static void CapSetName(Printer, Entry)
     Printer_t		       *Printer;
     char		       *Entry;
{
    char		       *End;
    static char			Buff[1024];
    char		      **Argv;
    int				Argc;
    int				i;

    /*
     * Entry looks like one of these:
     *	name:...
     *	name|alias:...
     *	name|alias|alias:...
     *
     * If "alias" has whitespace in it then consider it a descripiton.
     */

    /*
     * Make a copy of the Name section which we can mangle
     */
    End = strchr(Entry, ':');
    snprintf(Buff, sizeof(Buff), "%.*s", (End) ? End - Entry : 0, Entry);

    /* Parse the buffer */
    Argc = StrToArgv(Buff, "|", &Argv, NULL, 0);
    for (i = 0; i < Argc; ++i) {
	if (!Argv[i])
	    continue;
	if (i == 0) {
	    /* If it's first, it's the canonical queue name */
	    Printer->Queue = strdup(Argv[i]);
	} else if (strchr(Argv[i], ' ')) {
	    /* Whitespace so it must be a description */
	    Printer->Desc = strdup(Argv[i]);
	} else {
	    /* Assume it's a queue alias */
	    StrArrayAdd(&(Printer->Aliases),strdup( Argv[i]));
	}
    }
}

/*
 * Retrieve the value from Entry which has key Key.
 * Supports arbitrary length key's.
 */
static char *CapGetVal(Entry, Key)
     char		       *Entry;
     char		       *Key;
{
    char		       *ep;
    char		       *End;
    int				KeyLen;
    static char			Val[256];

    KeyLen = strlen(Key);

    for (ep = Entry; ep && *ep; ++ep) {
	/* Look for the start of an entry */
	if (*ep != ':')
	    continue;
	/* Advance past ':' */
	ep++;
	/* Find the end of this key */
	for (End = ep; 
	     End && *End && (*End != '=' && *End != ':' && *End != '\n');
	     ++End);
	if (KeyLen != End - ep) {
	    /* The length doesn't match */
	    ep = End;
	    continue;
	}
	if (strncmp(Key, ep, KeyLen) == 0) {
	    /* Advance to start of Value */
	    ep = End;
	    if (*ep == '=')
		++ep;
	    /* Find the end of the value */
	    End = strchr(End, ':');
	    /* It's a match! */
	    if (!End || !*End || (End-ep) == 0) {
		/* It's a boolean */
		return "1";
	    } else if (End && *End) {
		snprintf(Val, sizeof(Val), "%.*s", End-ep, ep);
		return Val;
	    }
	}
    }

    return (char *) NULL;
}

/*
 * Set a "bsdaddr=" string which looks like:
 *	server,remprinter,proto
 * This is a Solaris printers.conf feature.
 */
static void CapSetBsdaddr(Printer, String)
     Printer_t		       *Printer;
     char		       *String;
{
    static char			Buff[1024];
    char		      **Argv;
    int				Argc;

    snprintf(Buff, sizeof(Buff), "%s", String);
    Argc = StrToArgv(Buff, ",", &Argv, NULL, 0);
    if (Argc >= 1)
	Printer->Server = strdup(Argv[0]);
    if (Argc >= 2)
	Printer->RemoteQueue = strdup(Argv[1]);
    if (Argc >= 3) {
	if (EQ(Argv[2], "solaris"))
	    Printer->Proto = PP_LPD;
	else
	    SImsg(SIM_DBG, "CapSetBsdaddr: Unknown protocol <%s>", Argv[2]);
    }
}

/*
 * Check and set data found in /etc/lp/printers
 * This is a feature of Solaris 2.x
 */
static void CapSetEtcConfig(Printer)
     Printer_t		       *Printer;
{
    static char			Dir[MAXPATHLEN];
    static char			File[MAXPATHLEN];
    FILE		       *Fp;
    static char			Buff[1024];
    char		       *cp;
    char		       *Key;
    char		       *Val;
    char		      **Argv;
    int				Argc;
    int				i;

    if (!Printer || !Printer->Queue)
	return;

    snprintf(Dir, sizeof(Dir), "/etc/lp/printers/%s", Printer->Queue);
    snprintf(File, sizeof(File), "%s/configuration", Dir);

    if (Debug && FileExists(File)) {
	SImsg(SIM_DBG, "CapSetEtcConfig: BEGIN CONFIGURATION %s", File);
	snprintf(Buff, sizeof(Buff), "/bin/cat %s", File);
	system(Buff);
	SImsg(SIM_DBG, "CapSetEtcConfig: END CONFIGURATION %s", File);
    }

    Fp = fopen(File, "r");
    if (!Fp) {
	SImsg(SIM_DBG, "CapSetEtcConfig: %s open failed: %s", File, SYSERR);
	return;
    }

    while (fgets(Buff, sizeof(Buff), Fp)) {
	if (Buff[0] == '#')
	    continue;
	if (cp = strchr(Buff, '\n'))
	    *cp = CNULL;
	if (cp = strchr(Buff, ':'))
	    *cp = CNULL;
	else
	    continue;

	Key = Buff;
	/* Skip whitespace */
	for (Val = cp + 1; Val && *Val && *Val == ' '; ++Val);

	SImsg(SIM_DBG, "CapSetEtcConfig: Key=<%s> Val=<%s>",
	      PRTS(Key), PRTS(Val));

	if (!strlen(Val))
	    continue;

	/*
	 * Printer type: PS
	 */
	if (EQ(Key, "printer type")) {
	    if (EQ(Val, "PS"))
		StrArrayAdd(&(Printer->Langs), PL_PS);
	    else if  (EQ(Val, "PCL"))
		StrArrayAdd(&(Printer->Langs), PL_PCL);
	    else
	 	SImsg(SIM_DBG,
		      "CapSetEtcConfig: Printer type/lang <%s> unknown",
		      Val);
	    continue;
	}

	/*
	 * Content types: postscript,pcl
	 */
	if (EQ(Key, "content types")) {
	    Argc = StrToArgv(Val, ",", &Argv, NULL, 0);
	    for (i = 0; i < Argc; ++i) {
		if (EQ(Argv[i], "PS") || EQ(Argv[i], "postscript"))
		    StrArrayAdd(&(Printer->Langs), PL_PS);
		else
		    SImsg(SIM_DBG,
			  "CapSetEtcConfig: Printer content type <%s> unknown",
			  Argv[i]);
	    }
	    continue;
	}

	if (EQ(Key, "device")) {
	    if (!EQ(Val, "/dev/null"))
		Printer->Device = strdup(Val);
	} else {
	    /* Add whatever this is as a description item */
	    AddDesc(&(Printer->DescList), strdup(Key), strdup(Val), 0);
	}
    }

    fclose(Fp);

    /*
     * Now get the description out of the "comment" file
     */
    snprintf(File, sizeof(File), "%s/comment", Dir);

    if (Debug && FileExists(File)) {
	SImsg(SIM_DBG, "CapSetEtcConfig: BEGIN COMMENT %s", File);
	snprintf(Buff, sizeof(Buff), "/bin/cat %s", File);
	system(Buff);
	SImsg(SIM_DBG, "CapSetEtcConfig: END COMMENT %s", File);
    }

    Fp = fopen(File, "r");
    if (!Fp) {
	SImsg(SIM_DBG, "CapSetEtcConfig: %s open failed: %s", File, SYSERR);
	return;
    }
    if (fgets(Buff, sizeof(Buff), Fp)) {
	if (cp = strchr(Buff, '\n'))
	    *cp = CNULL;

	SImsg(SIM_DBG, "CapSetEtcConfig: Comment=<%s>", Buff);

	/*
	 * If Desc is not set or if what we just got is longer than 
	 * what we already had, then add it.
	 */
	if (!Printer->Desc || 
	    (Printer->Desc && strlen(Buff) > strlen(Printer->Desc))) {
	    if (Printer->Desc)
		(void) free(Printer->Desc);
	    Printer->Desc = strdup(Buff);
	}
    }

    fclose(Fp);
}

/*
 * Read printcap entries from File and add to PrinterTree
 */
static int CapFile(PrinterTree, File, Fp)
     Printer_t		      **PrinterTree;
     char		       *File;
     FILE		       *Fp;
{
    static char			Buff[2048];
    static Printer_t	       *Last;
    Printer_t		       *Printer;
    Desc_t		      **dp;
    char		       *cp;

    if (!PrinterTree || !File)
	return -1;

    SImsg(SIM_DBG, "CapFile: Reading from <%s>", File);

    while (CapGetEnt(Buff, sizeof(Buff), Fp) > 0) {
	SImsg(SIM_DBG, "CapFile: Entry=<%s>", Buff);

	if (EQN(Buff, "_default", 8))
	    continue;

	Printer = PrinterCreate(NULL);
	dp = &Printer->DescList;
	CapSetName(Printer, Buff);
	if (cp = CapGetVal(Buff, "bsdaddr")) 	/* Solaris printers.conf */
	    CapSetBsdaddr(Printer, cp);
	
	if (!Printer->Queue)
	    /* We should have this set by now */
	    continue;

	/*
	 * BSD/OS uses names starting with '.' as prototype templates
	 * which are not actual print queues.
	 */
	if (Printer->Queue[0] == '.') {
	    SImsg(SIM_DBG, "CapFile: %s: Skipping - queue starts with '.'",
		  Printer->Queue);
	    continue;
	}

	if (cp = CapGetVal(Buff, "description")) /* Solaris printers.conf */
	    Printer->Desc = strdup(cp);

	if (cp = CapGetVal(Buff, "rm"))
	    Printer->Server = strdup(cp);
	if (cp = CapGetVal(Buff, "rp"))
	    Printer->RemoteQueue = strdup(cp);

	if (cp = CapGetVal(Buff, "lp")) {
	    if (EQN(cp, "/dev", 4)) {
		Printer->Proto = PP_LOCAL;
		Printer->Device = strdup(cp);
	    } else if (REmatch(cp, ".*smb.*", NULL)) {
		/*
		 * The print filer is sometimes specified as
		 * 	lp=|/usr/share/printconf/util/smbprint
		 */
		Printer->Proto = PP_SMB;
	    }
	}

	if (cp = CapGetVal(Buff, "of"))
	    /*
	     * Look for "smb" in the output filter
	     */
	    if (REmatch(cp, ".*smb.*", NULL))
		Printer->Proto = PP_SMB;

	/*
	 * If no protocol has been set yet and there is a server
	 * then assume it's LPD.
	 */
	if (!Printer->Proto && Printer->Server)
	    Printer->Proto = PP_LPD;

	if (CapGetVal(Buff, "sh"))
	    AddDesc(dp, "Suppress Banners", "TRUE", 0);
	if (cp = CapGetVal(Buff, "mx"))
	    Printer->MaxJobSize = atol(cp);
	if (cp = CapGetVal(Buff, "sd"))
	    Printer->SpoolDir = strdup(cp);

	CapSetEtcConfig(Printer);		/* Solaris/SysV */

	SImsg(SIM_DBG, "CapFile: Queue=<%s> Server=<%s> Desc=<%s>",
	      PRTS(Printer->Queue), PRTS(Printer->Server),
	      PRTS(Printer->Desc));

	if (Printer->Queue)
	    PrinterAdd(PrinterTree, Printer);
    }

    return 0;
}

/*
 * Build Printer info from a "stock" LPR system.
 */
extern Opaque_t BuildPrinterLpr(PrinterTree, SearchExp)
     Printer_t		      **PrinterTree;
     char		      **SearchExp;
{
    int				Found = 0;
    char		      **File;
    char		        Cmd[1024];
    char		       *Ypcat;
    char		       *cp;
    FILE		       *Fp;
    int 			LongJumpVal;

    if (!PrinterTree)
	return (Opaque_t) -1;

    /*
     * Look in local file(s)
     */
    for (File = PrintCapFiles; File && *File; ++File) {
	if (Debug && FileExists(*File)) {
	    SImsg(SIM_DBG, "BuildPrinterLpr: BEGIN CAPFILE %s", *File);
	    snprintf(Cmd, sizeof(Cmd), "/bin/cat %s", *File);
	    system(Cmd);
	    SImsg(SIM_DBG, "BuildPrinterLpr: END CAPFILE %s", *File);
	}

	if (Fp = fopen(*File, "r")) {
	    if (CapFile(PrinterTree, *File, Fp) > 0)
		Found = 1;
	    
	    (void) fclose(Fp);
	} else {
	    SImsg(SIM_DBG, "BuildPrinterLpr: %s: Open for read failed: %s",
		  *File, SYSERR);
	}
    }

    /*
     * Look in NIS table
     */
    if (Ypcat = CmdFind(YpcatPath)) {
	snprintf(Cmd, sizeof(Cmd), "%s printers.conf.byname 2> /dev/null",
		 Ypcat);
	SImsg(SIM_DBG, "BuildPrinterLpr: Looking for printers with <%s>",
	      Cmd);

#if	defined(HAVE_SETJMP)
	/*
	 * Set a timeout since programs may hang
	 */
	if ((LongJumpVal = setjmp(Env)) != 0) {
	    /*
	     * If we reach this point then the time-out has occured
	     */
	    cp = SignalName(LongJumpVal);
	    SImsg(SIM_DBG,
		  "BuildPrinterLpr: Command <%s> terminated due to %s", 
		  Cmd, cp);
	    return (Opaque_t) Found;
	}
	TimeOutStart(60, TimeOutHandler);
#endif	/* HAVE_SETJMP */

	if (Debug) {
	    SImsg(SIM_DBG, "BuildPrinterLpr: BEGIN NIS %s", Cmd);
	    system(Cmd);
	    SImsg(SIM_DBG, "BuildPrinterLpr: END NIS %s", Cmd);
	}

	if (Fp = popen(Cmd, "r")) {
	    if (CapFile(PrinterTree, Cmd, Fp) > 0)
		Found = 1;

	    (void) pclose(Fp);
	} else {
	    SImsg(SIM_DBG, "BuildPrinterLpr: %s: Start command failed: %s",
		  Cmd, SYSERR);
	}

	TimeOutEnd();
    }

    return (Opaque_t) Found;
}
